# Generated by Django 3.1.7 on 2021-04-05 17:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("reports", "0001_initial"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Identifier",
        ),
    ]
